/****************************************************************************
  Project: NITRO-SampleTools - TXLib - TextureViewer
  File:    myLib_fontBgScr.c

  Copyright 2004,2005 Nintendo Co.,Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  This routine displays the font in BG.

  $Log: myLib_fontBgScr.c,v $
  Revision 1.8  2005/06/20 06:46:11  mominoki
  Updated copyright. Made revision adjustments. 


  Revision 1.1  2004/04/15		mominoki
  Fixed comments.

  Revision 1.0  2004/01/27		mominoki
  Entered initial version.

  $NoKeywords: $
******************************************************************************/
#include <nitro.h>

/******************************************************************************
	Definition
******************************************************************************/
#define BG_SC_PLTT_SHIFT        12

/******************************************************************************
	Prototype declaration
******************************************************************************/
void MyLib_FontBgScr_SetU8(
			u8 				figure,				// Figure to be displayed
			u16*			pDest,				// Destination address
			u16				colorIndex			// Color index
);

void MyLib_FontBgScr_SetChar(
	const	u8* 			pChar,					// Character strings to be displayed
			u16*			pDest,					// Destination address
			u16				colorIndex				// Color index
);

void MyLib_FontBgScr_SetU16(
			u16 			figure,					// Figure to be displayed
			u16*			pDest,					// Destination address
			u16				colorIndex,				// Color index
			u8 				sw						// Decimal (10) or hexadecimal (16)
);

/************************************************************************
	FontIntBgScreenSet()

	Set one numeric character in font BG screen

	Convert figure to character and transfer to VRAM
************************************************************************/
void MyLib_FontBgScr_SetU8(
			u8 				figure,				// Figure to be displayed
			u16*			pDest,				// Destination address
			u16				colorIndex			// Color index
)
{
	char* 	str;
	u8*	pChar;

	switch(figure){
		case 0:		str="0"; 	break;
		case 1:		str="1"; 	break;
		case 2:		str="2"; 	break;
		case 3:		str="3"; 	break;
		case 4:		str="4"; 	break;
		case 5:		str="5"; 	break;
		case 6:		str="6"; 	break;
		case 7:		str="7"; 	break;
		case 8:		str="8"; 	break;
		case 9:		str="9"; 	break;
		case 10:	str="A"; 	break;
		case 11:	str="B"; 	break;
		case 12:	str="C"; 	break;
		case 13:	str="D"; 	break;
		case 14:	str="E"; 	break;
		case 15:	str="F"; 	break;
		default: break;
	}

	pChar 	= (u8*)str;
	*pDest = (u16)(colorIndex << BG_SC_PLTT_SHIFT | *pChar);
}

/************************************************************************
	BgScSet()

	Set character strings in font BG screen
************************************************************************/
void MyLib_FontBgScr_SetChar(
	const 	u8* 			pChar,					// Character strings to be displayed
			u16*			pDest,					// Destination address
			u16				colorIndex				// Color index
)
{
	while (*pChar != '\0')
	{
		*pDest = 0;
		*pDest = (u16)(colorIndex << BG_SC_PLTT_SHIFT | *pChar);
		pDest++;
		pChar++;
	}
}

/************************************************************************
	BgIntRetuScSet()

	Set figure in font BG screen
************************************************************************/
void MyLib_FontBgScr_SetU16	(
			u16 			figure,					// Figure to be displayed
			u16*			pDest,					// Destination address
			u16				colorIndex,				// Color index
			u8 				sw						// Decimal (10) or hexadecimal (16)
)
{
	u8 	keta[5];
	u32 i;

	keta[0]	=	(u8)(figure /(sw * sw * sw * sw));
	keta[1]	=	(u8)((figure % (sw * sw * sw * sw)) / (sw * sw * sw));
	keta[2]	=	(u8)((figure % (sw * sw * sw)) / (sw * sw));
	keta[3]	=	(u8)((figure % (sw * sw)) / sw);
	keta[4]	=	(u8)( figure % sw );

	for(i=0; i<5; i++)
	{
		MyLib_FontBgScr_SetU8(keta[i], pDest + i, colorIndex);
	}
}
