/****************************************************************************
  Project: NITRO-SampleTools - TXLib
  File:    TXLib_ColorIndexImage.cpp

  Copyright 2004, 2005 Nintendo Co.,Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  Class handling color index component of CTexture   
  Do not use when CTexture is Direct.

  $Log: TXLib_ColorIndexImage.cpp,v $
  Revision 1.7  2005/06/20 06:54:11  mominoki
  Changed copyright. Made revision adjustments. 

  Revision 1.1  2004/04/15		mominoki
  Corrected comment

  Revision 1.0  2004/03/03		mominoki
  Initial registration

  $NoKeywords: $
 ***************************************************************************/
#define TLB_EXPORT_

#include <cstring>
#include <TXLib/Base/TXLib_ColorIndexImage.h>

namespace TXLib{

/*********************************************************************************************************
	TXLib::CColorIndexImage::CColorIndexImage

	Constructor
*********************************************************************************************************/
CColorIndexImage::CColorIndexImage()
:
	m_available(false),
	m_width(0),
	m_height(0),
	m_colorIndexData()
{
}

/*********************************************************************************************************
	TXLib::CColorIndexImage::

	Copy constructor
*********************************************************************************************************/
CColorIndexImage::CColorIndexImage
(
	const	CColorIndexImage&				rColorIndexImage			// CColorIndexImage of copy source
) :
	m_available			(rColorIndexImage.m_available		),
	m_width				(rColorIndexImage.m_width			),
	m_height			(rColorIndexImage.m_height			),
	m_colorIndexData	(rColorIndexImage.m_colorIndexData	)
{
}

/*********************************************************************************************************
	TXLib::CColorIndexImage::

	Assignment Operator
*********************************************************************************************************/
CColorIndexImage&
CColorIndexImage::operator=(
	const	CColorIndexImage&				rColorIndexImage				// CColorIndexImage of copy source
)
{
	if (&rColorIndexImage != this)
	{
		m_available			= rColorIndexImage.m_available;
		m_width				= rColorIndexImage.m_width;
		m_height			= rColorIndexImage.m_height;
		m_colorIndexData	= rColorIndexImage.m_colorIndexData;
	}
	return *this;
}

/*********************************************************************************************************
	TXLib::CColorIndexImage::

	Destructor
*********************************************************************************************************/
CColorIndexImage::~CColorIndexImage()
{ 
	Clear(); // Clear
} 

/*********************************************************************************************************
		private
*********************************************************************************************************/
/*********************************************************************************************************
	TXLib::CColorIndexImage::

	Memory Clear
*********************************************************************************************************/
void
CColorIndexImage::Clear()
{
	m_available = false;
	m_width = 0;
	m_height = 0;

	m_colorIndexData.Clear();
}

/*********************************************************************************************************
		public
*********************************************************************************************************/
/*********************************************************************************************************
	TXLib::CColorIndexImage::

	Create
*********************************************************************************************************/
void
CColorIndexImage::Create(
				u16							width,						// Image size width
				u16							height						// Image size height
)
{
	m_available = true;
	m_width     = width;
	m_height    = height;

	m_colorIndexData.Create( width * height, 0   );
}

/*********************************************************************************************************
	TXLib::CColorIndexImage::

	Get color index value
*********************************************************************************************************/
inline void
CColorIndexImage::GetValue(
				u8*							pData,						// Data pointer
				u16							x,						// X coordinate
				u16							y						// Y coordinate
) const
{
	assert(pData != NULL);

	m_colorIndexData.GetValue( pData, (y * m_width + x) );
}

/*********************************************************************************************************
	TXLib::CColorIndexImage::

	Set color index value
*********************************************************************************************************/
inline void
CColorIndexImage::SetValue(
				u8							data,						// Data index
				u16							x,							// X coordinate
				u16							y							// Y coordinate
)
{
	m_colorIndexData.SetValue( data, (y * m_width + x) );
}

} // namespace
