/****************************************************************************
  Project: NITRO-SampleTools - TXLib
  File:    TXLibUtility_SaveNtf.cpp

  Copyright 2004,2005 Nintendo Co.,Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  This sample function saves NTF.

  $Log: TXLibUtility_SaveNtf.cpp,v $
  Revision 1.8  2005/06/20 06:54:11  mominoki
  Revised copyright. Made revision adjustments. 

  Revision 1.1  2004/04/15		mominoki
  Fixed comments.

  Revision 1.0  2004/03/03		mominoki
  Entered initial version.

  $NoKeywords: $
 ***************************************************************************/
#include "stdafx.h"
#include <cstring>
#include <cstdio>
#include <cstdlib>
#include <time.h>
#include "../include/TXLibUtility.h"

namespace std{ using :: memset; }

namespace TXLibUtility{

/********************************************************************************************************
	TXLibUtility::SaveNtf_Binary()	

	Save NTF binary
*********************************************************************************************************/
static bool
SaveNtf_Binary( 
		const	char*							pPathName,				// Path name of save file
		const 	TXLib::CNtf::tMemory*			pNtfMemory				// Output data
)
{
	assert( ( pPathName != NULL ) && ( pNtfMemory != NULL ) );

	FILE*	pFile;
	char	pathName[256];

	// Palette
	if( ( pNtfMemory->pPaletteData != NULL ) && ( pNtfMemory->paletteDataByteSize > 0 ) )
	{
		strcpy( pathName, pPathName );													// Copy path name 
		if( !ChangeExtName( pathName, ".ntfp" ) ) return false;					// Append character string to file name and change extension
	
		if( ( pFile = fopen( pathName, "wb" ) ) == NULL ) return false;
		fwrite( pNtfMemory->pPaletteData, 1, pNtfMemory->paletteDataByteSize, pFile );	// Output data
		fclose(pFile);
	}

	// Palette index
	if( ( pNtfMemory->pPaletteIndexData != NULL ) && ( pNtfMemory->paletteIndexDataByteSize > 0) )
	{
		strcpy( pathName, pPathName );															// Copy path name 
		if( !ChangeExtName( pathName, ".ntfi" ) ) return false;						// Append character string to file name and change extension

		if( ( pFile = fopen( pathName, "wb" ) ) == NULL ) return false;
		fwrite( pNtfMemory->pPaletteIndexData, 1, pNtfMemory->paletteIndexDataByteSize, pFile ); // Output data
		fclose(pFile);
	}

	// Texel data
	if( ( pNtfMemory->pTexelData != NULL) && ( pNtfMemory->texelDataByteSize > 0 ) )
	{
		strcpy( pathName, pPathName );													// Copy path name 
		if( !ChangeExtName( pathName, ".ntft" ) ) return false;					// Append character string to file name and change extension

		if( ( pFile = fopen( pathName, "wb" ) ) == NULL ) return false;
		fwrite( pNtfMemory->pTexelData, 1,  pNtfMemory->texelDataByteSize, pFile );		// Output data
		fclose(pFile);
	}

	return true;
}

/********************************************************************************************************
	TXLibUtility::SaveNtf_Source()	

	Save NTF source
*********************************************************************************************************/
static bool
SaveNtf_Source( 
	const	char*							pPathName,						// Path name of save file
	const 	TXLib::CNtf::tMemory*			pNtfMemory,						// Output data
			u16								width,							// Width
			u16								height,							// Height
			u16								realWidth,						// Actual width
			u16								realHeight,						// Actual height
			TXLib::CTexture::tType			type,							// Conversion format
	const	char*							pInputPathName,					// Path name of input image file
			bool							textureViewFlg					// TextureViewerFlg
)
{
	assert( ( pPathName != NULL ) && ( pNtfMemory != NULL ) );

	FILE* pFile;

	char pathName[256];	// Path name
	char fileName[256]; // File name

	strcpy( pathName, pPathName ); // Copy path name 
	_splitpath( pPathName, NULL, NULL, fileName, NULL ); 			// Split the output path name and extract the file name.

	if( !ChangeExtName( pathName, ".c" ) ) return false; 			// Change extension

	if( ( pFile = fopen( pathName, "wb" ) ) == NULL ) return false; // Open file

	// Display information
	char	textureFormat[35];
	switch(type)
	{
		case TXLib::CTexture::kType_palette4:	strcpy( textureFormat, "GX_TEXFMT_PLTT4");	break;
		case TXLib::CTexture::kType_a5i3:		strcpy( textureFormat, "GX_TEXFMT_A5I3");	break;
		case TXLib::CTexture::kType_a3i5:		strcpy( textureFormat, "GX_TEXFMT_A3I5");	break;
		case TXLib::CTexture::kType_palette16:	strcpy( textureFormat, "GX_TEXFMT_PLTT16");	break;
		case TXLib::CTexture::kType_palette256:	strcpy( textureFormat, "GX_TEXFMT_PLTT256");	break;
		case TXLib::CTexture::kType_direct:		strcpy( textureFormat, "GX_TEXFMT_DIRECT");	break;
		case TXLib::CTexture::kType_tex4x4:		strcpy( textureFormat, "GX_TEXFMT_COMP4x4");	break;
		default: return false;
	}
	time_t utime;	// Time
	time(&utime);

	fprintf( pFile,"// NITRO Texture File\r\n");
	fprintf( pFile, "// format:     %s\r\n", textureFormat);
	fprintf( pFile, "// width:      %5d\r\n", width);
	fprintf( pFile, "// height:     %5d\r\n", height);
	fprintf( pFile, "// original_width:  %5d\r\n", realWidth);
	fprintf( pFile, "// original_height: %5d\r\n", realHeight);
	fprintf( pFile, "// date:       %s\r\n", ctime(&utime) );		// Time
	if( pInputPathName != NULL )
	{
		fprintf( pFile, "// source:     %s\r\n\r\n", pInputPathName );	// Input image file name
	}

	fputs( "#include <nitro.h>\r\n", pFile );						// "#include"

	if( textureViewFlg )
	{
		fputs( "#include \"textureDataTypedef.h\"\r\n\r\n", pFile );		// "#include"
	}

	// Palette
	char paletteName[ 256 ]; // Palette name
	if( pNtfMemory->paletteDataByteSize > 0 )
	{
		strcpy( paletteName, fileName );
		strcat( paletteName, "_Palette" ); // paletteName = fileName + "_Palette"

		u32 number = pNtfMemory->paletteDataByteSize / sizeof(u16);			// Number of data
		fprintf( pFile, "const u16 %s[ %d ] = {", paletteName, number ); // Front declaration
		for( u32 i=0; i<number; i++ )
		{
			if( !( i % 8 ) )		// 8 columns
			{
				fputs( "\r\n     ",pFile );	// Line feed
			}
			fprintf( pFile, "0x%04X, ", *(u16*)( pNtfMemory->pPaletteData + 2 * i ) ); // 2-byte data
		}
		fputs( "\r\n};\r\n\r\n", pFile );
	}

	// Palette index
	char paletteIndexName[ 256 ]; // Palette index name
	if( pNtfMemory->paletteIndexDataByteSize > 0)
	{
		strcpy( paletteIndexName, fileName );
		strcat( paletteIndexName, "_PaletteIndex" ); // paletteIndexName = fileName + "_PaletteIndex"

		u32 number = pNtfMemory->paletteIndexDataByteSize / sizeof(u16);						// Number of data

		fprintf( pFile, "const u16 %s[ (%d / 4) * (%d / 4)  ] = {", paletteIndexName, width, height ); // Front declaration

		for( u32 i=0; i<number; i++ )
		{
			if( !( i % 8 ) )		// 8 columns
			{
				fputs( "\r\n     ",pFile );	// Line feed
			}
			fprintf( pFile, "0x%04X, ", *(u16*)( pNtfMemory->pPaletteIndexData + 2 * i ) ); // 2-byte data
			if( ((i+1) % 8 ) == 0) // 8 columns
			{
				fprintf( pFile, " // %3d block line", i/(width/4) ); // Line number
			}
		}
		fputs( "\r\n};\r\n\r\n", pFile );
	}

	// Texel data
	char texelName[ 256 ]; // Texel name
	if( pNtfMemory->texelDataByteSize > 0 )
	{
		strcpy( texelName, fileName );
		strcat( texelName, "_Texel" ); // texelName = fileName + "_Texel"

		if( type == TXLib::CTexture::kType_tex4x4 ){
			u32 number = pNtfMemory->texelDataByteSize / sizeof(u32);									// Number of data

			fprintf( pFile, "const u32 %s[ (%d / 4) * (%d / 4)  ] = {", texelName, width, height ); // Front declaration
			for( u32 i=0; i<number; i++ )
			{
				if( !( i % 8 ) )		// 8 columns
				{
					fputs( "\r\n     ",pFile );	// Line feed
				}
				fprintf( pFile, "0x%08X, ", *(u32*)( pNtfMemory->pTexelData + 4 * i ) ); // 4-byte data
				if( ((i+1) % 8 ) == 0) // 8 columns
				{
					fprintf( pFile, " // %3d block line", i/(width/4) ); // Line number
				}
			}
			fputs( "\r\n};\r\n\r\n", pFile );

		}else if( type == TXLib::CTexture::kType_direct ){
			u32 number = pNtfMemory->texelDataByteSize / sizeof(u16);

			fprintf( pFile, "const u16 %s[ %d * %d ] = {", texelName, width, height ); // Front declaration
			for( u32 i=0; i<number; i++ )
			{
				if( !( i % 8 ) )		// 8 columns
				{
					fputs( "\r\n     ",pFile );	// Line feed
				}
				fprintf( pFile, "0x%04X, ", *(u16*)( pNtfMemory->pTexelData + 2 * i ) ); // 2-byte data
				if( ((i+1) % 8 ) == 0)
				{ // 8 columns
					fprintf( pFile, " // %3d line", i/width ); // Line number
				}
			}
			fputs( "\r\n};\r\n\r\n", pFile );
		}
		else
		{ // Color index
			u32 number = pNtfMemory->texelDataByteSize; // u8
			u8 bitCount;

			if( type ==  TXLib::CTexture::kType_palette4)
			{
				bitCount = 2;
			}
			else if(  type ==  TXLib::CTexture::kType_palette16 )
			{
				bitCount = 4;
			}
			else if( ( type ==  TXLib::CTexture::kType_a5i3) || ( type ==  TXLib::CTexture::kType_a3i5) || ( type ==  TXLib::CTexture::kType_palette256 )  )
			{
				bitCount = 8;
			}

			fprintf( pFile, "const u8 %s[ %d * %d / %d ] = {", texelName, width, height, ( 8 / bitCount ) ); // Front declaration
			for( u32 i=0; i<number; i++ )
			{
				if( !( i % 8 ) )		// 8 columns
				{
					fputs( "\r\n     ",pFile );	// Line feed
				}
				fprintf( pFile, "0x%02X, ", *( pNtfMemory->pTexelData + i ) ); // 1-byte data
				if( ((i+1) % 8 ) == 0)
				{ // 8 columns
					fprintf( pFile, " // %3d line", i/width ); // Line feed
				}
			}
			fputs( "\r\n};\r\n\r\n", pFile );
		}
	} // Texel


	// Display data for TextureViewer
	if( textureViewFlg )
	{
		fprintf( pFile, "const tTextureParameter		%s_Parameter = {\r\n", fileName );
		fprintf( pFile, "				%d,				%d,				%d,\r\n",
				pNtfMemory->texelDataByteSize, pNtfMemory->paletteDataByteSize, pNtfMemory->paletteIndexDataByteSize);
		fprintf( pFile, "				%s,	%d,			%d,		%d,		%d};\r\n\r\n",
			textureFormat, width, height, realWidth, realHeight);
		fputs( "//				Texel size, 		 Palette size, 		Palette index size\r\n", pFile );
		fputs( "//				Format, 				Width, 				Height, 	Actual width, 	Actual height\r\n\r\n", pFile );
	}


	fclose(pFile);

	return true;
}

/********************************************************************************************************
	TXLibUtility::SaveNtf()	

	Save NTF
*********************************************************************************************************/
bool
SaveNtf( 
	const 	char*					pPathName,					// Save file path name
			TXLib::CTexture*		pTexture,					// CTexture pointer
			bool					binaryFlg,					// Binary output flag ( 0: source, 1: binary)
	const 	char*					pInputPathName,				// Input file path name
			bool					textureViewFlg,				// TextureViewerFlg
			bool					sizeAlignmentFlg			// NTF size alignment flag
)
{
	assert( ( pPathName != NULL ) && ( pTexture != NULL ) );

	bool result = false; // Execution results

	TXLib::CNtf::tMemory ntfMemory;	// NTF output data
	std::memset( &ntfMemory, 0, sizeof(TXLib::CNtf::tMemory) ); // Clear

	// Get byte size required for NTF memory output.
	TXLib::CNtf ntf;
	if( !ntf.GetDataByteSize( 
									&ntfMemory.texelDataByteSize,
									&ntfMemory.paletteDataByteSize,
									&ntfMemory.paletteIndexDataByteSize,
									pTexture,
									sizeAlignmentFlg
									) ) return false;

	// Secure memory
	ntfMemory.pTexelData		= new u8[ ntfMemory.texelDataByteSize ];				// Texel data
	ntfMemory.pPaletteData		= new u8[ ntfMemory.paletteDataByteSize ];				// Palette data
	ntfMemory.pPaletteIndexData	= new u8[ ntfMemory.paletteIndexDataByteSize ];			// Palette index data

	// Save current texture size
	u16	realWidth	= pTexture->GetWidth();
	u16	realHeight	= pTexture->GetHeight();

	if( !ntf.Write( &ntfMemory, pTexture, sizeAlignmentFlg ) ) goto error; 	// Write CNtf

	if( binaryFlg )
	{
		if( !SaveNtf_Binary( pPathName, &ntfMemory ) ) goto error; // Save binary
	}
	else
	{
		if( !SaveNtf_Source(	pPathName,
								&ntfMemory,
								pTexture->GetWidth(),
								pTexture->GetHeight(),
								realWidth,
								realHeight,
								pTexture->GetType(),
								pInputPathName,
								textureViewFlg) ) goto error; // Save source
	}

	result = true;
error:
	// Clear NTF output data
	delete[]  ntfMemory.pPaletteData;
	ntfMemory.pPaletteData = NULL;
	delete[]  ntfMemory.pPaletteIndexData;
	ntfMemory.pPaletteIndexData = NULL;
	delete[]  ntfMemory.pTexelData;
	ntfMemory.pTexelData = NULL;

	return result;
}

} // namespace
