/****************************************************************************
  Project: NITRO-SampleTools - TXLib
  File:    FileTextureViewer.cpp

  Copyright 2004,2005 Nintendo Co.,Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 Create the header file and texture data file for TextureViewer.

  $Log: FileTextureViewer.cpp,v $
  Revision 1.8  2005/06/20 06:54:11  mominoki
  Updated copyright. Made revision adjustments. 

  Revision 1.1  2004/04/15		mominoki
  Fixed comments.

  Revision 1.0  2004/03/03		mominoki
  Entered initial version.

  $NoKeywords: $
 ***************************************************************************/
#include "stdafx.h"
#include <time.h>
#include <TXLib.h>
#include "../include/FileTextureViewer.h"

namespace std{ using :: memset; }

namespace TXLibUtilityEx{
/********************************************************************************************************
	TXLibUtilityEx::MakeNTFPartName()	

	Create the name of the NTF structure element from the file name
*********************************************************************************************************/
static void
MakeNTFPartName(
	const	char*						pFileName,						// File name
			char*						pTexelName,						// Texel name
			char*						pPaletteName,					// Palette name
			char*						pPaletteIndexName,				// Palette index name
			char*						pParameterName					// Parameter name
)
{
	assert( ( pFileName != NULL ) && ( pTexelName != NULL ) &&
		( pPaletteName != NULL ) && ( pPaletteIndexName != NULL) );

	// Create texel name
	std::memset( pTexelName, 0, _MAX_PATH );
	strcpy( pTexelName, pFileName );
	strcat( pTexelName, "_Texel" );

	// Create palette name
	std::memset( pPaletteName, 0, _MAX_PATH );
	strcpy( pPaletteName, pFileName );
	strcat( pPaletteName, "_Palette" );

	// Create palette index name
	std::memset( pPaletteIndexName, 0, _MAX_PATH );
	strcpy( pPaletteIndexName, pFileName );
	strcat( pPaletteIndexName, "_PaletteIndex" );

	// Create parameter name
	std::memset( pParameterName, 0, _MAX_PATH );
	strcpy( pParameterName, pFileName );
	strcat( pParameterName, "_Parameter" );
}

/********************************************************************************************************
	TXLibUtilityEx::SaveNtfHeaderFile()	

	Create and save NTF header file
*********************************************************************************************************/
bool
SaveNtfHeaderFile(
	const	char*						pPathName,						// Path name
	const	tTextureData*				pTextureDataArray,				// Texture data array
			u32							number							// Quantity
)
{
	assert( ( pPathName != NULL ) && ( pTextureDataArray != NULL ));

	// File write
	FILE* pFile;
	if( ( pFile = fopen( pPathName, "wb" ) ) == NULL ) return false; // Open file

	fputs( "#ifndef TEXTUREDATA_H_\r\n", pFile );
	fputs( "#define TEXTUREDATA_H_\r\n\r\n", pFile );
	fputs( "#include <nitro.h>\r\n", pFile );
	fputs( "#include \"textureDataTypedef.h\"\r\n\r\n", pFile );

	char fileName[_MAX_FNAME];

	for( u32 n=0; n<number; n++)
	{
		// Extract file name
		_splitpath( pTextureDataArray[n].ntfPathName, NULL, NULL, fileName, NULL );
		strcat( fileName, ".c" );

		bool existFlg = false;
		for( u32 nn=0; nn<n; nn++ )
		{
			if( strcmp( pTextureDataArray[n].ntfPathName, pTextureDataArray[nn].ntfPathName	) == 0 )
			{
				existFlg = true;
				break;
			}
		}

		if( !existFlg )
		{
			fprintf( pFile, "#include \"texture/%s\"\r\n", fileName );
		}
	}
	fputs("\r\n", pFile);

	fputs("#endif\r\n", pFile);

	fclose( pFile );

	return true;
}

/********************************************************************************************************
	TXLibUtilityEx::SaveNtfHeaderFile()	

	Create and save the NTF image parameter source file
*********************************************************************************************************/
bool SaveNtfImageParameterFile(
	const	char*						pPathName,						// Path name
	const	tTextureData*				pTextureDataArray,				// Texture data array
			u32							number							// Quantity
)
{
	assert( ( pPathName != NULL ) && ( pTextureDataArray != NULL ));

	// File write
	FILE* pFile;
	if( ( pFile = fopen( pPathName, "wb" ) ) == NULL ) return false; // Open file

	char lineChar[_MAX_PATH];

	fputs( "#include <nitro.h>\r\n", pFile );
	fputs( "#include \"textureData.h\"\r\n\r\n", pFile );
	fprintf(pFile, "u32 textureNumber = %d;\r\n\r\n",number );
	fprintf(pFile, "const tTextureData 	textureArray[ %d ] = {\r\n", number);

	char fileName[_MAX_FNAME];
	char texelName[_MAX_PATH];
	char paletteName[_MAX_PATH];
	char paletteIndexName[_MAX_PATH];
	char parameterName[_MAX_PATH];

	for(u32 n=0; n<number; n++)
	{
		// Extract file name
		_splitpath( pTextureDataArray[n].ntfPathName, NULL, NULL, fileName, NULL );

		// Create structural element name
		MakeNTFPartName( fileName, texelName, paletteName, paletteIndexName, parameterName );

		switch( pTextureDataArray[n].type )
		{
			case TXLib::CTexture::kType_palette4:
			case TXLib::CTexture::kType_a5i3:
			case TXLib::CTexture::kType_a3i5:
			case TXLib::CTexture::kType_palette16:
			case TXLib::CTexture::kType_palette256:
				sprintf( lineChar, "	{ %s,	%s,			NULL,		&%s},\r\n", texelName, paletteName, parameterName  );
				break;
			case TXLib::CTexture::kType_direct:
				sprintf( lineChar, "	{ (u8*)%s,	NULL,		NULL,		&%s},\r\n", texelName, parameterName  );
				break;
			case TXLib::CTexture::kType_tex4x4:
				sprintf( lineChar, "	{ (u8*)%s,	%s,	%s,	&%s},\r\n",
										texelName, paletteName, paletteIndexName, parameterName  );
				break;
			default:
				break;
		}
		fputs( lineChar, pFile );
	}


	fputs("};\r\n\r\n", pFile);

	fclose(pFile);

	return true;
}

} // namespace
