/****************************************************************************
  Project: NITRO-SampleTools - TXLib
  File:    TXLibUtilityEx_SaveBg.cpp

  Copyright 2004,2005 Nintendo Co.,Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  $Log: TXLibUtilityEx_SaveBg.cpp,v $
  Revision 1.7  2005/06/20 06:54:11  mominoki
  Updated copyright. Made revision adjustments. 

  Revision 1.3	2004/10/27		mominoki
  Added bgAlignmentPaletteNumberFlg to SaveBg() arguments. 
  Changed to allow selection of whether to output palette with 16 colors when 4 bit, and 256 colors when 8 bit, during BG output.  

  Revision 1.2	2004/07/28		mominoki
  Added offsetColorIndex to arguments of SaveBg_Source(),SaveBg().
  Added functionality that offsets color index values with palette16, palette256, affine, palette256x16.

  Revision 1.1	2004/04/15		mominoki
  Fixed comments.

  Revision 1.0	2004/03/03		mominoki
  Entered initial version.

  $NoKeywords: $
 ***************************************************************************/
#include "stdafx.h"
#include <time.h>
#include <TXLib.h>
#include "../../TXLibUtility/include/TXLibUtility.h"
#include "../include/TXLibUtilityEx_SaveBg.h"

namespace std{ using :: memset; }

namespace TXLibUtilityEx{

/********************************************************************************************************
	TXLibUtility::SaveNtf_Binary()	

	Save NTF binary
*********************************************************************************************************/
static bool
SaveBg_Binary( 
		const	char*							pPathName,			// Save file path name
		const 	TXLibUtilityEx::CBg::tMemory*	pBgMemory			// Output data
)
{
	assert( ( pPathName != NULL ) && ( pBgMemory != NULL ) );

	FILE*	pFile;
	char	pathName[256];

	// Palette data
	if( ( pBgMemory->pPaletteData != NULL ) && ( pBgMemory->paletteDataByteSize > 0 ) )
	{
		strcpy( pathName, pPathName );											// Copy path name 
		if( !TXLibUtility::ChangeExtName( pathName, ".nbfp" ) ) return false;	// Append character string to file name, and change extension.
	
		if( ( pFile = fopen( pathName, "wb" ) ) == NULL ) return false;
		fwrite( pBgMemory->pPaletteData, 1, pBgMemory->paletteDataByteSize, pFile );	// Output data
		fclose(pFile);
	}

	// Character data
	if( ( pBgMemory->pCharData != NULL ) && ( pBgMemory->charDataByteSize > 0) )
	{
		strcpy( pathName, pPathName );												// Copy path name 
		if( !TXLibUtility::ChangeExtName( pathName, ".nbfc" ) ) return false;		// Append character string to file name, and change extension.

		if( ( pFile = fopen( pathName, "wb" ) ) == NULL ) return false;
		fwrite( pBgMemory->pCharData, 1, pBgMemory->charDataByteSize, pFile );		// Output data
		fclose(pFile);
	}

	// Screen data
	if( ( pBgMemory->pScreenData != NULL) && ( pBgMemory->screenDataByteSize > 0 ) )
	{
		strcpy( pathName, pPathName );													// Copy path name 
		if( !TXLibUtility::ChangeExtName( pathName, ".nbfs" ) ) return false;			// Append character string to file name, and change extension.

		if( ( pFile = fopen( pathName, "wb" ) ) == NULL ) return false;
		fwrite( pBgMemory->pScreenData, 1,	pBgMemory->screenDataByteSize, pFile );	// Output data
		fclose(pFile);
	}

	return true;
}

/********************************************************************************************************
	TXLibUtilityEx::SaveBg_Source()	

	Save Bg source file
*********************************************************************************************************/
static bool
SaveBg_Source(
	const	char*						pPathName,						// Save file path name
	const	CBg::tMemory*				pBgMemory,						// Character data
			u16							width,							// Width
			u16							height,							// Height
			CBg::tType					bgType,							// Format
			bool						bgHFlip,						// H-Flip
			bool						bgVFlip,						// V-Flip
			bool						compressFlg,					// Character compression enabled ( kType_text_palette16, kType_text_palette256, kType_text_palette256X16 )
			u8							bgPaletteNumber,				// Palette number
			u8							offsetColorIndex,				// Color index offset value
	const	char*						pInputPathName					// Input image file path name
)
{
	assert( ( pPathName != NULL ) && ( pBgMemory != NULL ) );

	FILE* pFile;

	char pathName[256];	// Path name
	char fileName[256]; // File name

	strcpy( pathName, pPathName ); // Copy path name 
	_splitpath( pPathName, NULL, NULL, fileName, NULL ); 			// Split output path name, and extract file name.

	char* pChar = strrchr( pathName, '.' ); // Search for "." (extension) from the end

	if( pChar )
	{
		strcpy( pChar, ".c" );		// Change extension
	}
	else
	{
		strcat( pathName, ".c");	// Add extension
	}

	if( ( pFile = fopen( pathName, "wb" ) ) == NULL ) return false; // Open file


	// Display information
	char	bgTypeStr[35];
	switch(bgType)
	{
		case CBg::kType_text_palette16:			strcpy( bgTypeStr, "palette16");		break;
		case CBg::kType_text_palette256:		strcpy( bgTypeStr, "palette256");		break;
		case CBg::kType_affine:					strcpy( bgTypeStr, "affine");			break;
		case CBg::kType_exAffine_palette256x16:	strcpy( bgTypeStr, "palette256x16");	break;
		case CBg::kType_exAffine_palette256Bmp:	strcpy( bgTypeStr, "palette256Bmp");	break;
		case CBg::kType_exAffine_direct:		strcpy( bgTypeStr, "direct");		break;
		case CBg::kType_big256Bmp:				strcpy( bgTypeStr, "big256Bmp");			break;
		default: return false;
	}
	time_t utime;	// Time
	time(&utime);

	fprintf( pFile,"// NITRO Bg File\r\n");
	fprintf( pFile, "// format: %s\r\n", bgTypeStr);
	fprintf( pFile, "// width: %5d\r\n", width);
	fprintf( pFile, "// height: %5d\r\n", height);
	// Character BG information
	if( ( bgType == CBg::kType_text_palette16) || ( bgType == CBg::kType_text_palette256) ||
		( bgType == CBg::kType_affine) || ( bgType == CBg::kType_exAffine_palette256x16) )
	{
		// Palette number
		if( ( bgType == CBg::kType_text_palette16) || ( bgType == CBg::kType_exAffine_palette256x16) )
		{
			fprintf( pFile,"// palette number: %2d\r\n", bgPaletteNumber );
		}

		// Color index offset value
		fprintf( pFile,"// offset color index: %2d\r\n", offsetColorIndex );

		if( compressFlg )
		{
			// Flip
			if( bgType != CBg::kType_affine )
			{
				fprintf( pFile,"// Compress Character\r\n");

				if( bgHFlip )
				{
					fprintf( pFile,"// H-Flip: ON\r\n");
				}
				else
				{
					fprintf( pFile,"// H-Flip: OFF\r\n");
				}

				if( bgVFlip )
				{
					fprintf( pFile,"// V-Flip: ON\r\n");
				}
				else
				{
					fprintf( pFile,"// V-Flip: OFF\r\n");
				}
			}
		}
		else
		{
			fprintf( pFile,"// Does Not Compress Character\r\n");
		}
	}

	fprintf( pFile, "// date: %s\r\n", ctime(&utime) );		// Time
//	if( pInputPathName != NULL )
//	{
//		fprintf( pFile, "// source: %s\r\n\r\n", pInputPathName );	// Input image file name
//	}


	fputs( "#include <nitro.h>\r\n\r\n", pFile );					// "#include"

	// Palette
	char paletteName[ 256 ]; // Palette name
	if( pBgMemory->paletteDataByteSize > 0 )
	{
		strcpy( paletteName, fileName );
		strcat( paletteName, "_Palette" ); // paletteName = fileName + "_Palette"

		u32 number = pBgMemory->paletteDataByteSize / sizeof(u16);				// Number of data
		fprintf( pFile, "const u16 %s[ %d ] = {", paletteName, number ); // Front declaration
		for( u32 i=0; i<number; i++ )
		{
			if( !( i % 8 ) )		// 8 columns
			{
				fputs( "\r\n     ",pFile );	// Line feed
			}
			fprintf( pFile, "0x%04X, ", *(u16*)( pBgMemory->pPaletteData + 2 * i ) ); // 2-byte data
		}
		fputs( "\r\n};\r\n\r\n", pFile );
	}

	// Character data
	char charName[ 256 ]; // Character name
	if( pBgMemory->charDataByteSize > 0 )
	{
		strcpy( charName, fileName );
		strcat( charName, "_Char" ); // charName = fileName + "_Char"

		if(  ( bgType == CBg::kType_text_palette16 ) || ( bgType == CBg::kType_text_palette256 ) ||
			( bgType == CBg::kType_affine) || ( bgType == CBg::kType_exAffine_palette256x16))
		{
			u8	bitCount	= ( bgType ==  CBg::kType_text_palette16 ) ? 4 : 8;
			u8	blockByte	= ( bgType ==  CBg::kType_text_palette16 ) ? 32 : 64;

			fprintf( pFile, "const u8 %s[ %d ] = {", charName, pBgMemory->charDataByteSize ); // Front declaration

			u32 number = pBgMemory->charDataByteSize;
			for( u32 i=0, j=1; i<number; i++ )
			{
				if( !( i % bitCount ) )		// 8 columns
				{
					fputs( "\r\n     ",pFile );	// Line feed
				}
				fprintf( pFile, "0x%02X, ", *( pBgMemory->pCharData + i ) ); // 1-byte data
				if( ((i+1) % blockByte ) == 0)
				{ // 8 columns
					fprintf( pFile, " // %3d char\r\n", j-1 ); // Line feed
					j++;
				}
			}
			fputs( "\r\n};\r\n\r\n", pFile );
		}
	}

	// Screen data
	char	screenName[ 256 ];	// Screen name
	if( pBgMemory->screenDataByteSize > 0 )
	{
		strcpy( screenName, fileName );
		strcat( screenName, "_Screen" ); // screenName = fileName + "_Screen

		u16		charWidth	= width / 8;	// Width character count
		u16		charHeight	= height / 8;	// Height character count

		if( ( bgType == CBg::kType_text_palette16 ) || ( bgType == CBg::kType_text_palette256 ) ||
			( bgType == CBg::kType_exAffine_palette256x16))
		{
			fprintf( pFile, "const u16 %s[ %d * %d ] = {", screenName, charWidth, charHeight ); // Front declaration

			u32 number = charWidth * charHeight;
			for( u32 i=0; i<number; i++ )
			{
				if( !( i % 8 ) )		// 8 columns
				{
					fputs( "\r\n     ",pFile );	// Line feed
				}
				fprintf( pFile, "0x%04X, ", *(u16*)( pBgMemory->pScreenData + 2 * i ) ); // 2-byte data
			}
			fputs( "\r\n};\r\n\r\n", pFile );
		}
		else if( bgType == CBg::kType_affine )
		{
			fprintf( pFile, "const u8 %s[ %d * %d ] = {", screenName, charWidth, charHeight ); // Front declaration

			u32 number = charWidth * charHeight;
			for( u32 i=0; i<number; i++ )
			{
				if( !( i % 8 ) )		// 8 columns
				{
					fputs( "\r\n     ",pFile );	// Line feed
				}
				fprintf( pFile, "0x%02X, ", *(pBgMemory->pScreenData + i) ); // 1-byte data
			}
			fputs( "\r\n};\r\n\r\n", pFile );

		}
		else if( bgType == CBg::kType_exAffine_direct )
		{
			u32 number = pBgMemory->screenDataByteSize / sizeof(u16);

			fprintf( pFile, "const u16 %s[ %d * %d ] = {", screenName, width, height ); // Front declaration
			for( u32 i=0; i<number; i++ )
			{
				if( !( i % 8 ) )		// 8 columns
				{
					fputs( "\r\n     ",pFile );	// Line feed
				}
				fprintf( pFile, "0x%04X, ", *(u16*)( pBgMemory->pScreenData + 2 * i ) ); // 2-byte data
				if( ((i+1) % 8 ) == 0)
				{ // 8 columns
					fprintf( pFile, " // %3d line", i/width ); // Line number
				}
			}
			fputs( "\r\n};\r\n\r\n", pFile );
		}
		else if(( bgType == CBg::kType_big256Bmp ) || ( bgType == CBg::kType_exAffine_palette256Bmp ) )
		{
			u32 number = pBgMemory->screenDataByteSize;

			fprintf( pFile, "const u8 %s[ %d * %d ] = {", screenName, width, height ); // Front declaration
			for( u32 i=0; i<number; i++ )
			{
				if( !( i % 8 ) )		// 8 columns
				{
					fputs( "\r\n     ",pFile );	// Line feed
				}
				fprintf( pFile, "0x%02X, ", *( pBgMemory->pScreenData + i ) ); // 1-byte data
				if( ((i+1) % 8 ) == 0)
				{ // 8 columns
					fprintf( pFile, " // %3d linen", i/width ); // Line feed
				}
			}
			fputs( "\r\n};\r\n\r\n", pFile );
		}
	}

#if 0
	// Create variables for building into NITRO program
	fprintf( pFile, "// const tParameter		%s_Parameter = {\r\n", fileName );
	fprintf( pFile, "//				%d,				%d			%d,\r\n",
		pBgMemory->charDataByteSize, pBgMemory->paletteDataByteSize, pBgMemory->screenDataByteSize);
	fprintf( pFile, "//				%s,	      %d,				%d,};\r\n\r\n",	bgTypeStr, width, height);
	fputs( "//				Character size,		Palette size		Screen size\r\n",	pFile );
	fputs( "//				BG format,				Width,					Height\r\n",				pFile );
#endif

	fclose(pFile);
	return true;
}

/********************************************************************************************************
	TXLibUtilityEx::SaveBg()	

	Save NTF BG file
*********************************************************************************************************/
bool
SaveBg( 
			u32*					nChar,						// Text BG character count
	const 	char*					pPathName,					// Save file path name
			bool					binaryFlg,					// Binary output flag ( 0: source, 1: binary)
			TXLib::CTexture*		pTexture,					// CTexture pointer
			CBg::tType				bgType,						// Format
			bool					bgHFlip,					// H flip enabled ( kType_text_palette16, kType_text_palette256, kType_text_palette256X16 )
			bool					bgVFlip,					// V flip enabled ( kType_text_palette16, kType_text_palette256, kType_text_palette256X16 )
			bool					compressFlg,				// Character compression enabled ( kType_text_palette16, kType_text_palette256, kType_text_palette256X16 )
			u8						bgPaletteNumber,			// Palette number (kType_text_palette16, kType_text_palette256x16)
			bool					bgAlignmentPaletteNumberFlg, // Palette number alignment flag (kType_text_palette16, kType_text_palette256, kType_text_palette256X16)
			u8						offsetColorIndex,			// Color index offset value
	const	char*					pInputPathName				// Input image file path name
)
{
	assert( ( pPathName != NULL ) && ( pTexture != NULL ) );

	bool result = false; // Execution results

	CBg::tMemory	bgMemory;
	std::memset( &bgMemory, 0, sizeof(bgMemory) ); // Clear

	u16		width	= pTexture->GetWidth();
	u16		height	= pTexture->GetHeight();
	CBg		bg;

	// Get required memory size
	if(!bg.GetDataByteSizeMax(
			&bgMemory.charDataByteSize,			// Character size
			&bgMemory.paletteDataByteSize,			// Palette size
			&bgMemory.screenDataByteSize,			// Screen size
			width,									// Width
			height,									// Height
			bgType									// BG format
			)) return false;

	//Secure memory
	bgMemory.pCharData		= new u8[ bgMemory.charDataByteSize ];
	bgMemory.pPaletteData	= new u8[ bgMemory.paletteDataByteSize ];
	bgMemory.pScreenData	= new u8[ bgMemory.screenDataByteSize ];

	// Convert BG, write memory
	if( !bg.Write(	&bgMemory,
					pTexture,
					bgType,
					bgHFlip,
					bgVFlip,
					compressFlg,
					bgPaletteNumber,
					bgAlignmentPaletteNumberFlg,
					offsetColorIndex	)) goto error;

	// Output file
	if( binaryFlg )
	{
		if( !SaveBg_Binary(
					pPathName,								// Save file path name
					&bgMemory								// BG memory
					)) goto error;

	}
	else
	{
		// Output text
		if( !SaveBg_Source( 
					pPathName,								// Save file path name
					&bgMemory,								// BG memory
					width,									// Width
					height,									// Height
					bgType,									// Format
					bgHFlip,								// H-Flip
					bgVFlip,								// V-Flip
					compressFlg,							// Character count compression
					bgPaletteNumber,						// Palette number
					offsetColorIndex,						// Color index offset value
					pInputPathName							// Input image file
					) ) goto error;	
	}


	result = true;
error:
	// Copy text BG character count
	*nChar = bgMemory.charDataByteSize / TXLibUtilityEx::CBg::kDefine_block64;
	
	// Clear BG output data
	delete[]  bgMemory.pCharData;
	delete[]  bgMemory.pPaletteData;
	delete[]  bgMemory.pScreenData;

	return result;
}


} // namespace
