/****************************************************************************
  Project: NITRO-SampleTools - TXLib
  File:    MyColorProcessor.h

  Copyright 2004,2005 Nintendo Co.,Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  This is a class that conducts color processing. It is a function customized to only extract 0-4 colors
  from the block during tex4x4 conversion. All other functions are left to CDefaultColorProcessor.

  $Log: MyColorProcessor.h,v $
  Revision 1.8  2005/06/20 06:54:11  mominoki
  Changed copyright. Made revision adjustments. 

  Revision 1.1  2004/04/15		mominoki
  Corrected comment

  Revision 1.0  2004/03/03		mominoki
  Initial registration

  $NoKeywords: $
 ***************************************************************************/
#ifndef MYCOLORPROCESSOR_H_
#define MYCOLORPROCESSOR_H_

#include <TXLib.h>

class CMyColorProcessor :public TXLib::CDefaultColorProcessor
{
private:
	bool MakePalette_Tex4x4( 
				TXLib::CColorPalette*		pNewColorPalette,		// Newly created palette
		const	TXLib::CColorPalette* 		pOldColorPalette,		// Current color palette
				bool						transparencyFlg,		// Transparent color mode flag
				bool						linearFlg				// Linear interpolation flag
	); // Creates tex4x4 color palettes (0-4 colors) of high, medium, and low brightness.


	bool Get4Color_Tex4x4(
				TXLib::CColorPalette*		pColorPalette,					// Color Palette
				bool						transparencyFlg,				// Transparent color mode flag
		const	TXLib::CDirectColorImage*	pBlockDirectColorImage,			// 1 block image
		const	TXLib::CAlphaImage*			pBlockAlphaImage,				// 1 block alpha
				tAlgorithm					algorithm						// Algorithm
	); // Get a tex4x4 palette from one block.


public:
	CMyColorProcessor(){};									// Constructor
	virtual		~CMyColorProcessor(){};						// Destructor


	virtual bool ConvertToTex4x4(
		TXLib::CTexture*					pTexture					// Texture
	); 	// Convert from direct to tex4x4
};

#endif
