/****************************************************************************
  Project: NITRO-SampleTools - TXLib
  File:    MyCUtility.cpp

  Copyright 2004,2005 Nintendo Co.,Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

	Function group for use with C,C++

  $Log: MyCUtility.cpp,v $
  Revision 1.8  2005/06/20 06:54:11  mominoki
  Changed copyright. Made revision adjustments. 

  Revision 1.1  2004/04/15		mominoki
  Corrected comment

  Revision 1.0  2004/03/03		mominoki
  Initial registration

  $NoKeywords: $
 ***************************************************************************/
#include "stdafx.h"
#include "cassert"

namespace MyCUtility{
/****************************************************************************
	MyCUtility::IsCorrectPathName()	

	Determine whether path name is valid in C programs
****************************************************************************/
bool
IsCorrectPathName(
		char*						pPathName						// Path name
)
{
	assert(pPathName != NULL);

	char* pChar = pPathName;

	while( *pChar != NULL )
	{
		int charInt;
		charInt = (int)*pChar;

		if( isalnum( charInt ) == 0 )	// Alphabet
		{
			switch( charInt )
			{
				case '.':
				case '\\':
				case ':':
				case '_':
				case '/':
					break;
				default:
					return false;
			}
		}
		pChar++;
	}
	return true;
}

/****************************************************************************
	MyCUtility::IsCorrectExtName()	

	Determine whether path name's extension is correct 
        (i.e., whether it is the same as the specified extension)
	0: Same 1: Error (No extension) 2: Error (Wrong extention)
****************************************************************************/
int
IsCorrectExtName(
	const	char*			pPathName,				// Path name
	const	char*			pExtName				// Extention name
)
{
	assert( (pPathName != NULL) && (pExtName != NULL));

	char*	pChar;
	char	extName[_MAX_EXT ];		// Extention name

	pChar = strrchr( pPathName, '.');
	if(  pChar == NULL	)	return 1;

	strcpy( extName, pChar );		// Fetch extension name
	_strlwr( extName );				// Convert extension into lowercase
	if(strcmp( extName, pExtName ) != 0 ) return 2;

	return 0;
}

/****************************************************************************
	GetOptionColor()

	Read the color values inside 'option.'
	Color values take a range of 0(0x00) to 255(0xff).
	There is one token character.
	option is option names r,g,b.			Example: cp=0xff,0xff,0xff
	Error if there are two or more tokens.
****************************************************************************/
bool
GetOptionColor(
			unsigned char*			pRed,				// Color value Red
			unsigned char*			pGreen,				// Color value Green
			unsigned char*			pBlue,				// Color value Blue
	const	char*					pOptionAll,			// Entire option (color values included)
	const	char*					pToken				// Token character
)
{
	assert( (pRed != NULL) && (pGreen != NULL) && (pBlue != NULL) && (pOptionAll != NULL) );

	char	*pChar;

	// Are there two token characters?
	pChar = (char*)pOptionAll;
	int		count	= 0;
	size_t	n		= strlen( pOptionAll);
	for( unsigned int i=0; i<n; i++)
	{
		if( (int)*pChar == (int)*pToken )
		{
			count++;
		}
		pChar++;
	}
	if( count != 2 ) return false;

	// Get RGB element character string
	char*	pRgbChar[3];
	if( ( pRgbChar[0] = strtok( (char*)pOptionAll, pToken ) ) == NULL ) return false;
	pChar = pRgbChar[0];
	while( pChar != NULL )	// Skip to numerical value
	{
		if( isdigit((int)*pChar) > 0 ) break;
		pChar++;
	}
	pRgbChar[0] = pChar;	// Red
	if( ( pRgbChar[1] = strtok( NULL, pToken ) ) == NULL ) return false;			// Green
	if( ( pRgbChar[2] = strtok( NULL, pToken ) ) == NULL ) return false;			// Blue

	int		data[3];
	for( int i=0; i<3; i++)
	{
		data[i] = (int)strtol( pRgbChar[i], &pChar, 0);
		if( pChar == pRgbChar[i] ) return false;	// Not a numeric character 
		if( data[i] > 255 ) return false;				// Outside range
	}
	*pRed	= (unsigned char)data[0];
	*pGreen	= (unsigned char)data[1];
	*pBlue	= (unsigned char)data[2];

	return true;
}


} // namespace