/****************************************************************************
  Project: NITRO-SampleTools - TXLib
  File:    inputData.h

  Copyright 2004,2005 Nintendo Co.,Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

	* This is for acquiring input data from the options that have been input.

  $Log: inputData.h,v $
  Revision 1.8  2005/06/20 06:54:11  mominoki
  Changed copyright. Made revision adjustments. 

  Revision 1.2	2004/10/24		mominoki
  Added pBgAlignmentPaletteNumberFlg to GetInputBgData() arguments.
  During BG output, added ability to choose whether to output palette with 16 colors when 4 bit, and 256 colors when 8 bit.  

  Revision 1.1	2004/04/15		mominoki
  Revised comments. 

  Revision 1.0	2004/03/03		mominoki
  Initial registration. 

  $NoKeywords: $
 ***************************************************************************/
#ifndef INPUTDATA_H_
#define INPUTDATA_H_

#include <TXLib.h>
#include "ntexconv.h"

TXLib::CTexture::tType GetTextureType(
	const	char*				pTextureTypeName
); // Get the texture format from the texture format name.

TXLibUtilityEx::CBg::tType 
GetBgType(
	const	char*				pBgTypeName
); // Get the Bg format from the CBg format name.

tErrorStatus GetInputData( 
			char*							pInputPathName,				// Input image file path name
			char*							pOutputNtfPathName,			// Output NTF path name
			bool*							pNTFBinaryFlg,				// Output NTF binary format flag
			bool*							pNotAlignmentFlg,			// NTF size alignment flag (true: no alignment)
			char*							pOutputOptionPathName,		// Output option file path name
			char*							pPalettePathName,			// File path name used for the palette
			TXLib::CTexture::tType*			pConvertTextureType,		// Texture conversion format
			TXLib::CDefaultColorProcessor
				::tConvertFactor*			pFactor,					// Conversion factor
			_TCHAR*							argv[],						// Input data
	const	int*							pOption						// option location
); // Get data input from the console.

#ifdef USE_BG
tErrorStatus GetInputBgData( 
			char*							pOutputBgPathName,			// Output Bg path name
			bool*							pBgBinaryFlg,				// Output Bg binary format flag
			TXLibUtilityEx::CBg::tType*		pType,						// BG format
			u8*								pBgPaletteNumber,			// Palette number 
			bool*							pBgAlignmentPaletteNumberFlg, // Palette number adjustment flag
			u8*								pOffsetImage,				// Color index offset value 
			bool*							pVFlipFlg,					// VFlip flag (On: true, Off: false)
			bool*							pHFlipFlg,					// HFlip flag (On: true, Off: false)
			bool*							pCompressFlg,				// Character compression flag (On: true, Off: false)
	const	char*							pInputPathName,				// Input image file path name
			_TCHAR*							argv[],						// Input data
	const	int*							pOption						// option position
); // Gets the Bg data input from the console. 
#endif

#endif
