/****************************************************************************
  Project: NITRO-SampleTools - TXLib
  File:    TXLib_ByteData.h

  Copyright 2004,2005 Nintendo Co.,Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  This class records data of 1 byte or greater.

  $Log: TXLib_ByteData.h,v $
  Revision 1.7  2005/06/20 06:54:11  mominoki
  Updated copyright. Made revision adjustments. 

  Revision 1.1  2004/04/15		mominoki
  Revised the comments.

  Revision 1.0  2004/03/03		mominoki
  Initial registration.

  $NoKeywords: $
 ***************************************************************************/
#ifndef TXLIB_BYTEDATA_H_
#define TXLIB_BYTEDATA_H_

#include <cassert> 
#include <TXLib_Typedef.h>

namespace TXLib
{
#ifdef TLB_EXPORT_
	class __declspec(dllexport)CByteData
#else
	class __declspec(dllimport)CByteData
#endif
	{
	private:
		bool				m_available;		// Whether or not the data can be used
		u32					m_byteSize;			// Byte size
		u8					m_byteCount;		// Data byte width
		u32					m_number;			// Data quantity
		u8*					m_pData;			// Data
												
	public:
		CByteData();											// Default constructor

		CByteData( const CByteData&	rByteData);					// Copy constructor

		virtual ~CByteData();									// Destructor

		bool	IsAvailable()	const { return m_available; }
		u8		GetByteCount()	const { return m_byteCount; }			// Data byte width
		u32		GetNumber()		const { return m_number; }				// Data quantity
		u8*		GetpData()		const { return m_pData; }				// Gets data pointer


		void Clear();				// Clear


		void Create(											
					u8							byteCount,						// Data byte width
					u32							number,							// Data quantity
					u32							initialData						// Initial data value
		); // Creation


		inline void GetValue( 
					u32*						pData,							// Data pointer
					u32							index							// Index
		) const; // Gets the data


		inline void SetValue( 
					u32							data,							// Data
					u32							index							// Index
		); // Sets the data


		CByteData& operator=( const CByteData& rByteData);			// Substitute operator


		u8&	operator[](u32 index)
		{
			assert(index < m_byteSize);
			return m_pData[index];
		}


		const u8& operator[](u32 index) const
		{
			assert(index < m_byteSize);
			return m_pData[index];
		}
	};
};

#endif