/****************************************************************************
  Project: NITRO-SampleTools - TXLib
  File:    TXLib_Data.h

  Copyright 2004,2005 Nintendo Co.,Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  This class records data of 1 byte or greater.

  $Log: TXLib_Data.h,v $
  Revision 1.7  2005/06/20 06:54:11  mominoki
  Updated copyright. Made revision adjustments. 

  Revision 1.1  2004/04/15		mominoki
  Revised the comments.

  Revision 1.0  2004/03/03		mominoki
  Initial registration.

  $NoKeywords: $
 ***************************************************************************/
#ifndef TXLIB_DATA_H_
#define TXLIB_DATA_H_

#include <cassert> 
#include <TXLib_Typedef.h>

namespace TXLib
{
#ifdef TLB_EXPORT_
	class __declspec(dllexport)CData
#else
	class __declspec(dllimport)CData
#endif
	{
	private:
		bool				m_available;		// Whether or not the data can be used.
		u32					m_byteSize;			// Byte size
		u8*					m_pData;			// Data

	public:
		CData();							// Constructor
		CData(const CData& rData);			// Copy constructor

		virtual ~CData();					// Destructor

		void Clear();						// Memory clear

		bool		IsAvailable()	const { return m_available; }
		u8*			GetpData()		const { return m_pData; }				// Gets the data pointer
		u32			GetByteSize()	const { return m_byteSize; }			// Gets the byte size


		void Create(
					u32							byteSize,						// Byte size
					u8							initialData						// Initial data value
		); // Creation


		inline void GetValue( 
					u8*							pData,							// Data pointer
					u32							index							// Index
		) const; // Gets data


		inline void SetValue( 
					u8							data,							// Data
					u32							index							// Index
		) const; // Sets data


		void ChangeNumber(
					u32							byteSize						// Byte size that changes
		); // Changes byte size.


		void QuickSortTopDown(																				
					u32*						pIndex,							// Target data index
					u32							firstNumber,					// First data index number
					u32							lastNumber,						// Last data index number
					bool						topDown							// true: descending order  false: ascending order
		); // Data is sorted with the quick sort method.


		CData& operator=( const CData& rData);


		u8&	operator[](u32 index)
		{
			assert(index < m_byteSize);
			return m_pData[index];
		}

		const u8& operator[](u32 index) const
		{
			assert(index < m_byteSize);
			return m_pData[index];
		}
	};
};

#endif