/****************************************************************************
  Project: NITRO-SampleTools - TXLib
  File:    TXLib_ColorProcessor.h

  Copyright 2004,2005 Nintendo Co.,Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  This class conducts color processing. It is called from TXLib::CTexture.

  $Log: TXLib_ColorProcessor.h,v $
  Revision 1.7  2005/06/20 06:54:11  mominoki
  Updated copyright. Made revision adjustments. 

  Revision 1.1  2004/04/15		mominoki
  Revised the comments.

  Revision 1.0  2004/03/03		mominoki
  Initial registration.

  $NoKeywords: $
 ***************************************************************************/
#ifndef TXLIB_COLORPROCESSOR_H_
#define TXLIB_COLORPROCESSOR_H_

#include <TXLib_Typedef.h>
#include <TXLib/Base/TXLib_Data.h>
#include <TXLib/Base/TXLib_ByteData.h>
#include <TXLib/Base/TXLib_ColorPalette.h>
#include <TXLib/Base/TXLib_DirectColorImage.h>
#include <TXLib/Base/TXLib_AlphaImage.h>

namespace TXLib
{
	class CTexture;

#ifdef TLB_EXPORT_
	class __declspec(dllexport)CColorProcessor
#else
	class __declspec(dllimport)CColorProcessor
#endif
	{
	private:

	public:
		CColorProcessor(){};							// Constructor
		virtual		~CColorProcessor(){};				// Destructor


		virtual void SetConvertFactor(
			const	void*						pFactor					// Change coefficient
		) = 0;	// Sets the change coefficient.


		virtual int FindNearColor(
			const	CColorPalette*				pPalette,				// Search target palette
					u8							red,
					u8							green,
					u8							blue,
					s16							avoidNumber				// Index number that avoids transparent colors, etc.
		) = 0; 	// Searches for the nearest color


		virtual bool ConvertToPaletteType(
					CTexture*					pTexture
		) = 0; 	// Changes direct to the palette format texture


		virtual bool ConvertToTex4x4(
					CTexture*					pTexture
		) = 0; 	// Changes direct to tex4x4
	};
};

#endif